// MyPodConv.java
// Extended player can change pitch and tempo in realtime
// soundtouch.dll must reside in class directory
// or in system's search path, e.g. in c:\windows\system32

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import ch.aplu.util.*;

public class MyPodConv extends JFrame
  implements ActionListener, SoundPlayerListener
{
  private class MyTimerListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      showPos();
    }
  }

  private final JButton[] buttons =
  {
    new JButton("Open"),
    new JButton("Rewind"),
    new JButton("Stop"),
    new JButton("Play"),
    new JButton("Pause"),
    new JButton("Advance"),
    new JButton("Vol-"),
    new JButton("Vol+"),
    new JButton("Pitch-"),
    new JButton("Pitch+"),
    new JButton("Tempo-"),
    new JButton("Tempo+")
  };
  private final int nbButtons = buttons.length;
  private File soundFile = null;
  private SoundPlayerExt player = null;
  private volatile boolean isEOF;
  private int volume;
  private float totalPitchDelta;
  private float totalTempoDelta;
  private MySoundConverter soundConverter;

  public MyPodConv()
  {
    for (int i = 0; i < nbButtons; i++)
    {
      buttons[i].addActionListener(this);
      getContentPane().add(buttons[i]);
    }

    getContentPane().setLayout(new FlowLayout());
    setTitle("WAV/MP3 Sound Player with Pitch and Tempo Converter");
    setSize(500, 100);
    setLocation(200, 100);
    enableButtons(new boolean[]
      {
        true, false, false, false, false, false,
        false, false, false, false, false, false
      });
    Timer pollTimer = new Timer(1000, new MyTimerListener());
    pollTimer.start();
  }

   void enableButtons(boolean[] b)
  {
    for (int i = 0; i < b.length; i++)
      buttons[i].setEnabled(b[i]);
  }

  public void actionPerformed(ActionEvent evt)
  {
    isEOF = false;
    showPos();
    Object source = evt.getSource();
    if (source == buttons[0])  // Open
    {
      JFileChooser chooser = new JFileChooser();
      int rc = chooser.showOpenDialog(null);
      if (rc == JFileChooser.APPROVE_OPTION)
      {
        soundFile = chooser.getSelectedFile();
        player = new SoundPlayerExt(soundFile);
        player.addSoundPlayerListener(this);
        enableButtons(new boolean[]
          {
            true, true, true, true, true, true,
            true, true, true, true, true, true
          });
        soundConverter = new MySoundConverter(player.getFormat());
        player.addSoundConverter(soundConverter);
        totalPitchDelta = 0;
        totalTempoDelta = 0;
        volume = player.getVolume();
      }
    }
    if (source == buttons[1]) // Rewind
    {
      player.rewindTime(10000);
    }
    if (source == buttons[2])  // Stop
    {
      player.stop();
      enableButtons(new boolean[]
        {
          true, true, true, true, true, true,
          true, true, true, true, true, true
        });
    }
    if (source == buttons[3])  // Play
    {
      player.play();
      enableButtons(new boolean[]
        {
          false, true, true, true, true, true,
          true, true, true, true, true, true
        });
    }
    if (source == buttons[4])  // Pause
    {
      player.pause();
    }
    if (source == buttons[5])  // Advance
    {
      player.advanceTime(10000);
    }
    if (source == buttons[6])  // Volume-
    {
      if (player != null)
      {
        volume -= 50;
        if (volume < 0)
          volume = 0;
        player.setVolume(volume);
      }
    }
    if (source == buttons[7])  // Volume+
    {
      if (player != null)
      {
        volume += 50;
        if (volume > 1000)
          volume = 1000;
        player.setVolume(volume);
      }
    }
    if (source == buttons[8])  // Pitch-
    {
      if (player != null)
      {
        totalPitchDelta -= 1;
        soundConverter.setParams(totalPitchDelta, totalTempoDelta);
      }
    }
    if (source == buttons[9])  // Pitch+
    {
      if (player != null)
      {
        totalPitchDelta += 1;
        soundConverter.setParams(totalPitchDelta, totalTempoDelta);
      }
    }
    if (source == buttons[10])  // Tempo-
    {
      if (player != null)
      {
        totalTempoDelta -= 1;
        soundConverter.setParams(totalPitchDelta, totalTempoDelta);
      }
    }
    if (source == buttons[11])  // Tempo+
    {
      if (player != null)
      {
        totalTempoDelta += 1;
        soundConverter.setParams(totalPitchDelta, totalTempoDelta);
      }
    }
  }

  private void showPos()
  {
    if (player != null)
    {
      if (isEOF)
      {
        setTitle("End of resource");
        enableButtons(new boolean[]
          {
            true, true, true, true, true, true,
            true, true, true, true, true, true
          });
      }
      else
      {
        double time = player.getCurrentTime() / 1000 / (1 + totalTempoDelta / 100);
        setTitle("Pos: " + (int)time + " s" +
          " - Vol: " + player.getVolume() + " of 1000" +
          " - Pitch: " + (int)totalPitchDelta + " halftone(s)" +
          " - Tempo: " + (int)totalTempoDelta + " percent(s)");
      }
    }
  }

  public void notifySoundPlayerStateChange(int reason, int mixerNb)
  {
    if (reason == 4)  // End of resource
      isEOF = true;
  }

  protected void processWindowEvent(WindowEvent e)
  {
    if (e.getID() == WindowEvent.WINDOW_CLOSING)
    {
      if (player != null)
        player.stop();
      System.exit(0);
    }
  }

  public static void main(String[] args)
  {
    EventQueue.invokeLater(new Runnable()
    {
      public void run()
      {
        new MyPodConv().setVisible(true);
      }
    });
  }
}
